from django.contrib.auth.decorators import login_required
from django.utils.decorators import method_decorator
from django.utils.timezone import now
from django.views.generic import ListView, DeleteView


class SessionMixin(object):
    def get_queryset(self):
        return self.request.user.session_set\
            .filter(expire_date__gt=now()).order_by('-last_activity')


class LoginRequiredMixin(object):
    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        return super(LoginRequiredMixin, self).dispatch(request, *args, **kwargs)


class SessionListView(LoginRequiredMixin, SessionMixin, ListView):
    def get_context_data(self, **kwargs):
        kwargs['session_key'] = self.request.session.session_key
        return super(SessionListView, self).get_context_data(**kwargs)


class SessionDeleteView(LoginRequiredMixin, SessionMixin, DeleteView):
    pass



from collections import OrderedDict
import re
import warnings

from django import template
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _, ugettext

try:
    from django.contrib.gis.geoip import GeoIP
    geoip = GeoIP()
except Exception as e:
    warnings.warn(str(e), stacklevel=2)
    geoip = None


register = template.Library()

BROWSERS = OrderedDict((
    (re.compile('Safari'), _('Safari')),
    (re.compile('Firefox'), _('Firefox')),
    (re.compile('Chrome'), _('Chrome')),
    (re.compile('Opera'), _('Opera')),
    (re.compile('MSIE'), _('Internet Explorer')),
))
DEVICES = OrderedDict((
    (re.compile('iPhone'), _('iPhone')),
    (re.compile('iPad'), _('iPad')),
    (re.compile('(Macintosh|OS_X)'), _('Mac OS X')),
    (re.compile('Apple'), _('Apple')),
    (re.compile('Android'), _('Android')),
    (re.compile('NT 5.1'), _('Windows XP')),
    (re.compile('NT 6.0'), _('Windows Vista')),
    (re.compile('NT 6.1'), _('Windows 7')),
    (re.compile('NT 6.2'), _('Windows 8')),
    (re.compile('Windows'), _('Windows')),
    (re.compile('Linux'), _('Linux')),
))


@register.filter
def humanagent(value):
    for regex, name in BROWSERS.items():
        if regex.search(value):
            browser = name
            break
    else:
        browser = 'unknown'
    for regex, name in DEVICES.items():
        if regex.search(value):
            device = name
            break
    else:
        device = 'unknown'
    return _('%s on %s') % (browser, device)


@register.filter
def location(value):
    if not geoip:
        return ''
    location = geoip.city(value)
    if location and location['country_name']:
        if location['city']:
            return '%s, %s' % (location['city'], location['country_name'])
        else:
            return location['country_name']
    else:
        return mark_safe('<i>%s</i>' % ugettext('unknown'))



from distutils.core import setup
from setuptools import find_packages

setup(
    name='django-user-sessions',
    version='0.1.0-dev',
    author='Bouke Haarsma',
    author_email='bouke@webatoom.nl',
    packages=find_packages(exclude=('demo', 'tests',)),
    package_data={
        'user_sessions': ['templates/user_sessions/*.html'],
    },
    url='http://github.com/Bouke/django-user-sessions',
    description='User sessions for django',
    license='MIT',
    long_description=open('README.rst').read(),
    install_requires=['Django>=1.5'],
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Topic :: Security',
    ],
)
